from crono import job, triggers

# Triggers

def on(*args, **kwargs):
	return job.Job(trigger=triggers.date(*args, **kwargs))

def after(*args, **kwargs):
	return job.Job(trigger=triggers.timer(*args, **kwargs))

def every(*args, **kwargs):
	return job.Job(trigger=triggers.interval(*args, **kwargs))

def at(*args, **kwargs):
	raise Exception('`at` trigger not implemented')

def cron(*args, **kwargs):
	return job.Job(trigger=triggers.cron(*args, **kwargs))

# Tasks

def log(*args, **kwargs):
	return job.Job(task='crono.tasks.log', args=args, kwargs=kwargs)

def request(*args, **kwargs):
	return job.Job(task='crono.tasks.request', args=args, kwargs=kwargs)

def message(*args, **kwargs):
	return job.Job(task='crono.tasks.message', args=args, kwargs=kwargs)

def email(*args, **kwargs):
	return job.Job(task='crono.tasks.email', args=args, kwargs=kwargs)



import requests
import logging

from crono import queue

@queue.queue.task
def log(level, msg, *args, **kwargs):
	logging.log(level, msg, *args, **kwargs)

@queue.queue.task
def message(*args, **kwargs):
	raise Exception('`message` task not implemented')

@queue.queue.task
def request(method, url, **kwargs):
	requests.request(method, url, **kwargs)

@queue.queue.task
def email(*args, **kwargs):
	raise Exception('`email` task not implemented')




def seconds(hours=None, minutes=None, seconds=None):
	time = None

	if seconds:
		time = seconds

	if minutes:
		time = (time if time else 0) + (minutes * 60)

	if hours:
		time = (time if time else 0) + (hours * 60 * 60)

	return time



import os
import uuid
import redbeat
import logging

from crono import queue, triggers


class Job:

	def __init__(self, trigger=None, task=None, args=None, kwargs=None):
		self.trigger = trigger
		self.task = task
		self.args = args
		self.kwargs = kwargs

	def save(self):

		if self.task != None and self.trigger != None:
			self.name = str(uuid.uuid4())
			self.entry = redbeat.schedulers.RedBeatSchedulerEntry(name=self.name, task=self.task, schedule=self.trigger, args=self.args, kwargs=self.kwargs, app=queue.queue)
			self.entry.save()
			logging.debug(self.entry)

		return self

	# def delete(self):
	# 	entry = redbeat.schedulers.RedBeatSchedulerEntry.from_key('redbeat:{}'.format(self.key), app=queue.queue)
	# 	entry.delete()
	# 	return entry

	# Triggers

	def on(self, *args, **kwargs):
		self.trigger = triggers.date(*args, **kwargs)
		return self.save()

	def after(self, *args, **kwargs):
		self.trigger = triggers.timer(*args, **kwargs)
		return self.save()

	def every(self, *args, **kwargs):
		self.trigger = triggers.interval(*args, **kwargs)
		return self.save()

	def at(self, *args, **kwargs):
		raise Exception('`at` trigger not implemented')

	def cron(self, *args, **kwargs):
		self.trigger = triggers.cron(*args, **kwargs)
		return self.save()

	# Tasks

	def log(self, *args, **kwargs):
		self.task = 'crono.tasks.log'
		self.args = args
		self.kwargs = kwargs
		return self.save()

	def request(self, *args, **kwargs):
		self.task = 'crono.tasks.request'
		self.args = args
		self.kwargs = kwargs
		return self.save()

	def message(self, *args, **kwargs):
		self.tas = 'crono.tasks.message'
		self.args = args
		self.kwargs = kwargs
		return self.save()

	def email(self, *args, **kwargs):
		self.task = 'crono.tasks.email'
		self.args = args
		self.kwargs = kwargs
		return self.save()



import os
import celery

# NOTE
# Name of the main module if running as __main__.
# This is used as the prefix for auto-generated task names.
queue = celery.Celery('crono')

queue.conf.include = ['crono.tasks']
queue.conf.broker_url = os.getenv('CELERY_BROKER')
queue.conf.result_backend = os.getenv('CELERY_RESULT_BACKEND')
queue.conf.beat_scheduler = 'redbeat.schedulers.RedBeatScheduler'
queue.conf.broker_pool_limit = int(os.getenv('CELERY_BROKER_POOL_LIMIT', 0))
queue.conf.task_ignore_result = bool(os.getenv('CELERY_TASK_IGNORE_RESULT', True))
queue.conf.beat_max_loop_interval = int(os.getenv('CELERY_BEAT_MAX_LOOP_INTERVAL', 300))

# NOTE
# Heroku Redis' Hobby Dev has a limit of 20 connections
# https://elements.heroku.com/addons/heroku-redis
queue.conf.redis_max_connections = int(os.getenv('REDIS_MAX_CONNECTIONS', 20))

# NOTE
# Celery seems to have a memory leak with redis broker
# http://docs.celeryproject.org/en/3.1/configuration.html#celeryd-max-tasks-per-child
# http://docs.celeryproject.org/en/latest/userguide/configuration.html#worker-max-memory-per-child
queue.conf.worker_max_tasks_per_child = int(os.getenv('CELERY_WORKER_MAX_TASKS_PER_CHILD', 100))

if __name__ == '__main__':
	queue.start()



import datetime
import redbeat
import celery

from crono import queue, utils

def timer(hours=None, minutes=None, seconds=None):
	seconds_ = utils.seconds(hours=hours, minutes=minutes, seconds=seconds)
	start = datetime.datetime.utcnow() + datetime.timedelta(seconds=seconds_)
	return redbeat.schedules.rrule(redbeat.schedules.SECONDLY, dtstart=start, count=1, app=queue.queue)

def date(datetime_):
	return redbeat.schedules.rrule(redbeat.schedules.SECONDLY, dtstart=datetime_, count=1, app=queue.queue)

def interval(hours=None, minutes=None, seconds=None):
	seconds_ = utils.seconds(hours=hours, minutes=minutes, seconds=seconds)
	return celery.schedules.schedule(run_every=seconds_, app=queue.queue)

def cron(expression):
	minute, hour, day_of_week, day_of_month, month_of_year = expression.split(' ')
	return celery.schedules.crontab(
		minute=minute,
		hour=hour,
		day_of_week=day_of_week,
		day_of_month=day_of_month,
		month_of_year=month_of_year,
		app=queue.queue)
